/*-------------------------------

	DISK II Emulator (2 of 2)

	(C) 2015 Koichi Nishida
	
-------------------------------*/
/*
Copyright (C) 2016 Koichi NISHIDA
email to Koichi NISHIDA: tulip-house@msf.biglobe.ne.jp

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include <avr/io.h>

#define RDR VPORT2_OUT,PIN0_bp
#define SENSER VPORT2_OUT,PIN3_bp

.global DISK2_prepare
.global DISK2_doBuffering
.global DISK2_writePtr
.global DISK2_readPulse
.global DISK2_posBit
.global DISK2_ptrByte
.global DISK2_byteData
.global buffer1
.global TCC1_OVF_vect
.global PORTD_INT0_vect
.global PROT_VAL
.global DISK2_buf_end

TCC1_OVF_vect:

	push	r18
	in		r18,_SFR_IO_ADDR(SREG)
	push	r18

	ldi		r18,0b00000001
	sts		TCC1_INTFLAGS, r18

	; check write protect
	in		r18,VPORT0_IN	; phase
	lsr		r18
	andi	r18,1
	lds		r19,PROT_VAL
	or		r19,r18
	breq	SENSE_OFF
	sbi		SENSER
	rjmp	SENSE_MARGE
SENSE_OFF:
	cbi		SENSER
	nop
SENSE_MARGE:

	; prepare check
	lds		r18,DISK2_prepare
	cpi		r18,1
	brne	TC_NOSKIP

	pop		r18
	out		_SFR_IO_ADDR(SREG),r18	
	pop		r18
	reti

TC_NOSKIP:
	; EN1/EN2 check (moved to DISK2.c) 

;	in		r18,VPORT2_IN
;	andi	r18,(PIN5_bm|PIN6_bm)
;	cpi		r18,(PIN5_bm|PIN6_bm)
;	breq	TC_SKIP

	lds		r18,DISK2_readPulse

	sbrc	r18,0				; 1/2/3
	sbi		RDR					; X1M2

	push	R19					; X1M2
	push	R26					; X1M2
	push	R27					; X1M2

	nop
	ldi		r26,8
1:
	dec		r26
	brne	1b

	cbi		RDR					; X1M2

	lds		r18,DISK2_posBit
	dec		r18
	sts		DISK2_posBit,r18

	brne	ROLBYTE

	ldi		r18,8
	sts		DISK2_posBit,r18

	lds		r27,DISK2_ptrByte+1
	lds		r26,DISK2_ptrByte

	ld		r18,X
	sts		DISK2_byteData,r18

	adiw	r26,1
	sts		DISK2_ptrByte+1,r27
	sts		DISK2_ptrByte,r26

	lds		r18,DISK2_buf_end+1
	cp		r27,r18
	brne	ROLBYTE

	lds		r18,DISK2_buf_end
	cp		r26,r18
	brne	ROLBYTE

	ldi		r18,1
	sts		DISK2_prepare,r18

	ldi		r18,lo8(buffer1)
	sts		DISK2_ptrByte,r18
	ldi		r18,hi8(buffer1)
	sts		DISK2_ptrByte+1,r18

ROLBYTE:
	lds		r18, DISK2_byteData
	ldi		r19,0
	rol		r18
	sts		DISK2_byteData,r18

	rol		r19

	sts		DISK2_readPulse,r19
	pop		R27
	pop		R26
	pop		r19
TC_SKIP:
	pop		r18
	out		_SFR_IO_ADDR(SREG),r18	
	pop		r18
	reti

; WREQ interrupt

PORTD_INT0_vect:

	push	r18
	in		r18, _SFR_IO_ADDR(SREG)
	push	r18

	ldi r18,0
	sts DISK2_magState,r18
	
	in		r18,VPORT2_IN
	andi	r18,(PIN5_bm|PIN6_bm)
	cpi		r18,(PIN5_bm|PIN6_bm)
	brne	ENABLE
	jmp		NOT_ENABLE
ENABLE:

	push	r19						; X1M2
	lds		r19,DISK2_magState		; 2

	; wait start bit 1
1:
	in		r18,VPORT0_IN			; 1
	andi	r18,64					; 1

	eor		r18,r19					; 1
	breq	1b						; 2/1

	; from here 192

	in		r18,VPORT0_IN			; 1
	andi	r18,64					; 1

	sts		DISK2_magState,r18		; X3M2

	ldi		r18, 16					; 1   *
1:									;     * 48
	dec		r18						; 1	  *
	brne	1b						; 1/2 *

	nop								; 1

	push	r20						; X1M2
	push	r21						; X1M2
	push	r22						; X1M2
	push	r23						; X1M2
	push	r24						; X1M2
	push	r30						; X1M2
	push	r31						; X1M2
	ldi		r22,0					; 1 start storing
	lds		r30,(DISK2_writePtr)	; X3M2
	lds		r31,(DISK2_writePtr+1)	; X3M2
	ldi		r19,lo8(349)			; 1
	ldi		r20,hi8(349)			; 1 
	rjmp	ENTR					; 2

WLP2:
	lds		r21,DISK2_magState		; X3M2

	; wait start bit 1
1:

	in		r23,VPORT0_IN			; 1
	andi	r23,64					; 1
	eor		r23,r21					; 1
	breq	1b						; 2/1

	; from here 192 clock

	in		r23,VPORT0_IN			; 1
	andi	r23,64					; 1

	sts		DISK2_magState,r23		; X3M2

	ldi		r23, 	24				; 1 *
1:									;   * 72
	dec		r23						; 1 *
	brne	1b						; 2 *

ENTR:
	ldi		r18,7					; 1
	ldi		r24,1					; 1

	; loop : 128 clock
WLP1:

	in		r23,VPORT2_IN			; 1
	andi	r23,4					; 1
	brne	WRITE_END				; 1
		
	nop								; 1
	nop								; 1
	ldi		r23, /* 36 adjust */35	; 1 *

1:									;   * 108
	dec		r23						; 1 *
	brne	1b						; 2 *

	in		r23,VPORT0_IN			; 1
	andi	r23,64					; 1

	lds		r21,DISK2_magState		; 2
	sts		DISK2_magState,r23		; 2
	eor		r23,r21					; 1
	lsr		r23						; 1

	lsr		r23						; 1
	swap	r23						; 1

	lsl		r24						; 1
	or		r24,r23					; 1
	dec		r18						; 1
	brne	WLP1					; 2/1

	; detect first D5

	cpi		r24,0xD5				; 1
	brne	NOT_START				; 2/1
	ldi		r22,1					; 1
NOT_START:
	cpi		r22,0					; 1
	breq	WLP2					; 2/1

	st		Z+,r24					; X1M2
	subi	r19,1					; 1
	sbci	r20,0					; 1
	brne	WLP2					; 2/1

WRITE_END:
	ldi		r21,1
	sts		DISK2_doBuffering,r21
	pop		r31
	pop		r30
	pop		r24
	pop		r23
	pop		r22
	pop		r21
	pop		r20
	pop		r19

NOT_ENABLE:
	sbi VPORT2_INTFLAGS, PORT_INT0IF_bm;
	pop		r18
	out		_SFR_IO_ADDR(SREG), r18	
	pop		r18
	reti