﻿/*
 * MOUNT.c
 *
 * Created: 2013/10/10 21:26:42
 *  Author: Koichi Nishida
 */
/*
Copyright (C) 2016 Koichi NISHIDA
email to Koichi NISHIDA: tulip-house@msf.biglobe.ne.jp

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "COMMON.h"
#include "MOUNT.h"
#include "SD.h"
#include "LCD.h"
#include "BUFFER.h"
#include "INI.h"
#include "DISK2.h"
#include "SMART.h"
#include "FLASHAIR.h"
#include <string.h>

// properties
volatile int8_t MOUNT_running = 0;

// initialize user interface
void MOUNT_init(void)
{
	MOUNT_clearProperties();
}

// clear properties
void MOUNT_clearProperties(void)
{
	MOUNT_running = 0;
}

// write back to the DSK image
void MOUNT_writeBack(uint8_t drive)
{
	uint8_t isDsk2 = (mode == DISK2MODE);
	
	OFF_TIMER;
	
	if (isDsk2 && !WP) {
		uint8_t err = 0;
		INI_read(buffer2.ini.ini, &err);
		if (err) { return; }
		if (buffer2.disk2.img[drive].valid) {
			uint8_t err = 0;

			FILE_openAbs(&buffer2.ini.img[drive], (char *)buffer2.ini.ini+drive*64, 0, &err);
			if (!err && !buffer2.ini.img[drive].protect) {
				DISK2_img_processing[drive] = WRITING_BACK;
				if (isFlashAir) FLASHAIR_set_status();
				DISK2_nic2Dsk(&buffer2.ini.img[drive], &buffer2.disk2.img[drive], &err);
				if (!err) buffer2.disk2.img[drive].written = 0;
				DISK2_img_processing[drive] = 0;
			}
		}
	}
}

// mount an image in the FlashAir to the drive
void MOUNT_mount(uint8_t drive)
{
	uint8_t err = 0;
	uint8_t isDsk2 = (mode == DISK2MODE);
		
	OFF_TIMER;
	MOUNT_running = 1;

#ifdef AIRDISP
	LCD_cls();
	LCD_locate(0,1);
	LCD_print((char *)buffer2.wifi.short_name+(strlen((const char*)buffer2.wifi.short_name)-11), 8);
#endif
	INI_substitute(buffer2.ui.filelist/*this is just buffer*/, isDsk2?drive:(drive+2), (char *)buffer2.ui.fullpath, &err);
	if (memcmp(buffer2.ui.fullpath, "           ",11) == 0) {
		if (isDsk2)  buffer2.disk2.img[drive].valid = 0; else buffer2.smart.img[drive].valid = 0; 
	} else {
		FILE_openAbs((isDsk2?&buffer2.ui.img[drive]:&buffer2.smart.img[drive]), (char *)buffer2.ui.fullpath, 0, &err);
		if (isDsk2) {
			cli();
			if (!err) {
				FILE_substituteFullpathExtwith((char *)buffer2.ui.fullpath, "NIC");
				FILE_openAbs(&buffer2.disk2.img[drive], (char *)buffer2.ui.fullpath, 0, &err);
				if (err) {
					if (!WP) {
						err = 0;
						DISK2_img_processing[drive] = NIC_CONVERTING;
						if (isFlashAir) FLASHAIR_set_status();	
#ifdef AIRDISP
						LCD_printAll("CreatingNIC file");
#endif
						// Notice : buffer2.sd.buf[512] is also used!
						FILE_createAbs(&buffer2.disk2.img[drive], (char *)buffer2.ui.fullpath, 286720, &err);
						if (!err) {
							FILE_openAbs(&buffer2.disk2.img[drive], (char *)buffer2.ui.fullpath, 0, &err);
							buffer2.disk2.img[drive].protect = buffer2.ui.img[drive].protect;	
							DISK2_dsk2Nic(&buffer2.disk2.img[drive], &buffer2.ui.img[drive], 0xfe, &err);
							if (err) buffer2.disk2.img[drive].valid = 0;
						}
						DISK2_img_processing[drive] = 0;
					} else {
#ifdef AIRDISP
						LCD_printAll("Write   protect.");
#endif				
					}
				}
			} else buffer2.disk2.img[drive].valid = 0;
			sei();
		} else {
			SMART_is2mg[drive] = (!err && buffer2.smart.img[drive].valid && (memcmp(buffer2.smart.img[drive].name+8, "2MG", 3)==0));
		}
	}
	MOUNT_running = 0;
}

// operation for exchanging the SD card
// mode / isFlashAir are set here!
uint8_t MOUNT_exchangeSD(uint8_t start)
{
	uint8_t err = 0;

	if (EJECT || start) {
		for (uint8_t drv = 0; drv < 4; drv++) buffer2.smart.img[drv].valid = 0;
		SD_p.inited = 0;
		MOUNT_clearProperties();
	}
	if (!SD_detect(start)) return 0;

	isFlashAir = IS_FLASHAIR();
	mode = GET_MODE();
	uint8_t isDsk2 = (mode == DISK2MODE);
	uint8_t drvNum = (isDsk2?2:4);

	if (isFlashAir) {
		cli();
		INI_openCreate(&err);
		sei();
		if (err) { return 0; }
	
		for (uint8_t drv = 0; drv < drvNum; drv++) {
			uint8_t n = (isDsk2?drv:(drv+2));
			err = 0;
			cli();
			INI_read(buffer2.ini.ini, &err);
			if (err) { sei(); return 0; }
			FILE_openAbs((isDsk2?(&buffer2.ini.img[drv]):(&buffer2.smart.img[drv])), (char *)buffer2.ini.ini+n*64, 0, &err);
			sei();
			if (err) { if (isDsk2) buffer2.disk2.img[drv].valid=0; else buffer2.smart.img[drv].valid = 0; continue; }

			#ifdef AIRDISP
			LCD_locate(0,1);
			if (isDsk2) {
				LCD_print((char *)buffer2.disk2.img[drv].name, 8);
			} else {
				LCD_print((char *)buffer2.smart.img[drv].name, 8);
			}
			#endif

			err = 0;
			if (isDsk2) {
				FILE_substituteFullpathExtwith((char *)buffer2.ini.ini+n*64, "NIC");
				cli();
				FILE_openAbs(&buffer2.disk2.img[drv], (char *)buffer2.ini.ini+n*64, 0, &err);
				sei();
				if(!err) { continue; }
				err = 0;
				if (!WP) {
					cli();
					#ifdef AIRDISP
					LCD_printAll("CreatingNIC file");
					#endif
					// Notice : buffer2.sd.buf[512] is also used!
					FILE_createAbs(&buffer2.disk2.img[drv], (char *)buffer2.ini.ini+n*64, 286720, &err);
					if (err) { buffer2.disk2.img[drv].valid=0; sei(); continue; }
					FILE_openAbs(&buffer2.disk2.img[drv], (char *)buffer2.ini.ini+n*64, 0, &err);
					if (err) { buffer2.disk2.img[drv].valid=0; sei(); continue; }
					buffer2.disk2.img[drv].protect = buffer2.ini.img[drv].protect;
					DISK2_dsk2Nic(&buffer2.disk2.img[drv], &buffer2.ini.img[drv], 0xfe, &err);
					if (err) { buffer2.disk2.img[drv].valid=0; sei(); continue; }
					sei();
				} else {
					#ifdef AIRDISP
					LCD_printAll("Write   protect.");
					#endif
				}
				sei();
			} else 	SMART_is2mg[drv] = (!err && buffer2.smart.img[drv].valid && (memcmp(buffer2.smart.img[drv].name+8, "2MG", 3)==0));
		}
	} else {
		if (isDsk2) {
			for (uint8_t drive = 0; drive < 2; drive++) {
				uint8_t err = 0;
				
				struct FILE directory;
				char ext_[4]="   ", name_[9]="D       ";
				name_[1] = ('1'+drive);
				cli();
				FILE_open(&directory, 0, name_, ext_, 1, &err);
				sei();
				if (err) continue;
				cli();
				FILE_open(&buffer2.disk2.img[drive], directory.startCluster, 0, "NIC", 1, &err);
				sei();
				if (err) continue;

				#ifdef AIRDISP
					LCD_locate(0,1);
					LCD_print((char *)buffer2.disk2.img[drive].name, 8);
				#endif
			}
		} else {
			for (uint8_t drive = 0; drive < 4; drive++) {
				uint8_t err = 0;
			
				struct FILE directory;
				char ext_[4]="   ", name_[9]="S       ";
				name_[1] = ('1'+drive);
				cli();
				FILE_open(&directory, 0, name_, ext_, 1, &err);
				sei();
				if (err) continue;
			
				cli();
				FILE_open(&buffer2.smart.img[drive], directory.startCluster, 0, "PO ", 1, &err);
				sei();
				if (err) continue;

				#ifdef AIRDISP
				LCD_locate(0,1);
				LCD_print((char *)buffer2.smart.img[drive].name, 8);
				#endif
			}
		}
	}
	return 1;
}