﻿/*
 * main
 *
 * Created: 2013/09/21 20:00:03
 *  Author: Koichi Nishida
 */ 
/*
Copyright (C) 2016 Koichi NISHIDA
email to Koichi NISHIDA: tulip-house@msf.biglobe.ne.jp

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <string.h>
#include <util/delay.h>
#include "COMMON.h"
#include "SYSTEM.h"
#include "LCD.h"
#include "SD.h"
#include "DISK2.h"
#include "SMART.h"
#include "MOUNT.h"
#include "INI.h"
#include "BUFFER.h"
#include "FLASHAIR.h"
#include "MESSAGE.h"

int main(void)
{
	CCP = CCP_IOREG_gc;
	NVM_CTRLB |= NVM_SPMLOCK_bm;

	system_clocks_init();
	COMMON_init();

#ifdef AIRDISP
	LCD_init();
#endif

#ifdef DEBUG
	MESSAGE_init();
#endif
	MOUNT_init();

	// show the type to the CPLD
	switch (AIR_TYPE) {
		case AIR_20:
		case AIR_C:
			PORTD.OUTSET = PIN1_bm;
			PORTD.DIRSET = PIN1_bm;
			break;
		case AIR_19:
			break;
	}

	if (FIRMWRITTEN) {
#ifdef AIRDISP
		LCD_locate(0,0);
		LCD_printAll("FirmwareWritten!");
#endif
		uint8_t err = 0;
		SD_init(&err);

		// TCD0 about 1 sec used for FlashAir
		TCD0.PER = 31250;
		TCD0.CTRLA = TC0_CLKSEL2_bm|TC0_CLKSEL1_bm|TC0_CLKSEL0_bm;			// prescaler : clk/1024
		TCD0.INTCTRLA = 0;
	
		mode = FIRMMODE;
		int16_t cnt = 0;
		sei();
		while (1) {
			int16_t new_cnt = TCD0_CNT;
			if (((new_cnt - cnt) < 0)) {
				if (FLASHAIR_command()) {}
			}
			cnt = new_cnt;
			_delay_ms(10);
		}
		while (1);
	}
	
	MOUNT_exchangeSD(1);

	sei();
	switch (mode) {
		case DISK2MODE:
			DISK2_init();
			DISK2_run();
		case SMARTMODE:
			SMART_init();
			SMART_run();
		case FIRMMODE:
			while (1) ;
	}
}